@@ -30,13 +30,12 @@ override('prepare', sub {
 
     $self->{KEYCOUNT} = 0;
     foreach my $ds (@{ $datasets }) {
-        if($ds->max_key_count() > $self->{KEYCOUNT}) {
-            $self->{KEYCOUNT} = $ds->max_key_count();
+        $self->{SCOUNT} += $ds->count;
+        if($ds->max_key_count > $self->{KEYCOUNT}) {
+            $self->{KEYCOUNT} = $ds->max_key_count;
         }
     }
 
-    $self->{SCOUNT} = 1;
-
     return 1;
 });
 
@@ -50,58 +49,62 @@ override('draw', sub {
     my $width = $self->width();
 
     my $dses = $clicker->get_datasets_for_context($self->context);
-    my $dscount = scalar(@{ $dses });
 
     my $padding = $self->bar_padding + $self->stroke->width;
 
-    if(!$self->{BWIDTH}) {
-        $self->{BWIDTH} = int(($width / $self->{KEYCOUNT}) / $dscount / 2);
-    }
-
-    if(!$self->{XOFFSET}) {
-        $self->{XOFFSET} = int((($self->{BWIDTH} + $padding) * $dscount) / 2);
-    }
+    my $bwidth = int(($width / $self->{KEYCOUNT})) - $self->stroke->width;
+    my $hbwidth = int($bwidth / 2);
 
+    my $offset = 1;
     foreach my $ds (@{ $dses }) {
         foreach my $series (@{ $ds->series }) {
-
             # TODO if undef...
             my $ctx = $clicker->get_context($ds->context);
             my $domain = $ctx->domain_axis;
             my $range = $ctx->range_axis;
 
-            my @vals = @{ $series->values() };
-            my @keys = @{ $series->keys() };
+            # Fudge amounts change mess up the calculation of bar widths, so
+            # we compensate for them here.
+            my $cbwidth = ($bwidth - ($bwidth * $domain->fudge_amount)) / $self->{SCOUNT};
+            my $chbwidth = int($cbwidth / 2);
 
             my $color = $clicker->color_allocator->next();
 
-            # Calculate the bar width we can use to fit all the datasets.
-
             my $base = $range->baseline();
             my $basey;
             if(defined($base)) {
                 $basey = $height - $range->mark($base);
             } else {
                 $basey = $height;
-                $base = $range->range->lower();
+                $base = $range->range->lower;
             }
 
-            my $sksent = $series->key_count() - 1;
-            for(0..$sksent) {
-                # Add the series_count times the width to so that each bar
-                # gets rendered with it's partner in the other series.
-                my $x = $domain->mark($keys[$_]) + ($self->{'SCOUNT'} * $self->{'BWIDTH'});
+            my @vals = @{ $series->values() };
+            my @keys = @{ $series->keys() };
+
+            my $sksent = $series->key_count;
+            for(0..($sksent - 1)) {
+                # I don't remember why all this math is here, but it works
+                # perfectly now. ;)
+                my $x = $domain->mark($keys[$_]);
                 my $y = $range->mark($vals[$_]);
 
                 if($vals[$_] >= $base) {
-                    $cr->rectangle(
-                        ($x + $padding) - $self->{'XOFFSET'}, $basey,
-                        - ($self->{'BWIDTH'} - $padding), -int($y - ($height - $basey)),
-                    );
+                    if($self->{SCOUNT} == 1) {
+                        $cr->rectangle(
+                            $x + $chbwidth , $basey,
+                            -int($cbwidth), -int($y - ($height - $basey))
+                        );
+                    } else {
+                        $cr->rectangle(
+                            $x - $hbwidth + ($offset * $cbwidth), $basey,
+                            -int($cbwidth), -int($y - ($height - $basey))
+                        );
+                    }
                 } else {
                     $cr->rectangle(
-                        ($x + $padding) - $self->{'XOFFSET'}, $basey,
-                        - ($self->{'BWIDTH'} - $padding), int($height - $basey - $y),
+                        $x - $hbwidth + ($offset * $cbwidth), $basey,
+                        -int($cbwidth), int($height - $basey - $y)
                     );
                 }
             }
@@ -125,13 +128,15 @@ override('draw', sub {
             $cr->set_source_rgba($color->as_array_with_alpha());
             $cr->stroke();
 
-            $self->{'SCOUNT'}++;
+            $offset++;
         }
     }
 
     return 1;
 });
 
+__PACKAGE__->meta->make_immutable;
+
 no Moose;
 
 1;
